clear all
close all
clc

%% General constants
R=8.314;
phi=0.57; % Transition packing fraction
%% Liquid's reference data
% van der Waals density
M=100.202e-3; % Molar mass
vdWd=(M/0.07849)*1e3; % vdW density (kg/m3) taking vdW molar volume from DIPPR
%% Reference saturated data (at atmospheric pressure)
datasat=csvread('NISTsat_heptane.txt');
Ts0=datasat(:,1);
rhos0=datasat(:,3);
Cps0=datasat(:,9)*1e3;
cs0=datasat(:,10);
% Fitting functions
p=3;%Order of polynomial
[coefrho{1},~,coefrho{2}]=polyfit(Ts0,rhos0,p);
rhos=@(T) polyval(coefrho{1},T,[],coefrho{2});
% Speed of sound fitting
[coefc{1},~,coefc{2}]=polyfit(Ts0,cs0,p);
cs=@(T) polyval(coefc{1},T,[],coefc{2});
% Heat capacity fitting
[coefcp{1},~,coefcp{2}]=polyfit(Ts0,Cps0,p);
Cps=@(T) polyval(coefcp{1},T,[],coefcp{2});
% Isobaric expansion coefficient
alphaPs=@(T) -(polyval(polyder(coefrho{1}),T,[],coefrho{2})/coefrho{2}(2))./rhos(T);
% Isothermal compressibility
kappaTs=@(T) (1./(cs(T).^2)+T.*alphaPs(T).^2./Cps(T))./rhos(T);
% log(kappaT) fitting
[coeflogkT{1},~,coeflogkT{2}]=polyfit(Ts0,log(kappaTs(Ts0)),p);
%% Calculation of the FT-EoS's parameter
drhos=@(T) polyval(polyder(coefrho{1}),T,[],coefrho{2})/coefrho{2}(2);
dlogkTs=@(T) polyval(polyder(coeflogkT{1}),T,[],coeflogkT{2})/coeflogkT{2}(2);
k=@(T) (-1./rhos(T)-(1./T+dlogkTs(T))./drhos(T));

%% Experimental data
%% Melikhov
data2=csvread('heptane_Melikhov.txt');
Pe2=data2(2:end,1);
Te2=data2(1,2:end);
rhoe2=data2(2:end,2:end);

%% Output temparatures and densities
P=Pe2;
T=Te2;

% Transition density and pressure
rhotr=phi*vdWd;
Ptr=@(T) (exp(k(T).*(rhotr-rhos(T)))-1)./(k(T).*rhos(T).*kappaTs(T))*1e-6;
% Weighting function
w=@(P,T) 0.5*(atan(phi*(P-Ptr(T))/Ptr(T))/(pi/2)+1);

pkg load statistics

for j=1:20;
    ph(j)=0.55+0.005*j;
    [rhoa,kTa]=fmdkT(P,T,coefrho,coeflogkT,vdWd,ph(j)); 
    AADm=100*abs(rhoe2-rhoa)./rhoe2;
    AAD(j)=nanmedian(AADm(:));
end
[vj,nj]=min(AAD)

[rhoa,kTa,ca]=fmdkTs(P,T,coefrho,coeflogkT,coefc,vdWd,ph(nj));  %

rhotr=ph(nj)*vdWd
ph(nj)
